//**************************************************************************
// Header File Included Area
//**************************************************************************
.include GPL951_Body.inc


//***************************************************************************************
// External Variable Declaration
//***************************************************************************************
.external R_Dac1Data
.external R_Dac2Data


//**************************************************************************
// Contant Defintion Area
//**************************************************************************
.define C_DebounceCnt		0x0010
.define C_SACM_RAMP_DELAY   80
.define C_SCAN_8_Bits	    1 //0 for 16Bits

		
//**************************************************************************
// Variable Publication Area
//**************************************************************************


//**************************************************************************
// Function Call Publication Area
//**************************************************************************
.public  _System_Initial
.public F_System_Initial
.public  _System_ServiceLoop
.public F_System_ServiceLoop
.public _WatchdogClear
.public F_WatchdogClear
.public  _SP_GetCh
.public F_SP_GetCh

.public _F_SYS_Initial
.public	F_SYS_CounterEnable
.public F_SYS_CounterDisable
.public F_SYS_GetCount
.public	_F_SYS_CounterEnable
.public _F_SYS_CounterDisable
.public _F_SYS_GetCount

.public  _Delay1ms
.public F_Delay1ms

//**************************************************************************
// RAM Definition Area
//**************************************************************************
System_RAM: .section .ram
.var R_DebounceReg
.var R_DebounceCnt
.var R_KeyBuf
.var R_KeyStrobe

.public R_128HzCounter

.var	R_128HzCounter					//Record 128Hz INT counter value

//**************************************************************************
// CODE Definition Area
//**************************************************************************
.CODE

//=======================================================================================
// Function name: F_SYS_Initial
// Purpose		: System Initial
// Parameter	: 
// Return		: 
// Destroy		: 
//=======================================================================================
F_SYS_Initial:
_F_SYS_Initial:
	
	call F_SYS_CounterEnable
	r1=0
	[R_128HzCounter]=r1
	
	retf


//=======================================================================================
// Function name: F_SYS_CounterEnable
// Purpose		: Enable 128Hz INT
// Parameter	: 
// Return		: 
// Destroy		: r1
//=======================================================================================
F_SYS_CounterEnable:
_F_SYS_CounterEnable:
	r1=(C_TimeBaseCFlag+C_TimeBaseCIntEn+C_TimeBaseCEn+C_TimeBaseC_128Hz)
	[P_TimeBaseC_Ctrl]=r1		
	irq on	
	retf


//****************************************************************
// Function    : F_System_Initial
// Description : System initial setting
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
 _System_Initial: .proc
F_System_Initial:
	int off;
	fir_mov off;
//	r1=C_PLL_Fast_En+C_SYSCLK_SRC_Div2	//48MHz	
//	[P_Clock_Ctrl]=r1
	call	F_System_Initial_User_IO

	retf;
	.endp;

//****************************************************************
// Function    : F_Key_Scan_Initial
// Description : Key scan variable initialzation
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
F_Key_Scan_Initial:	.proc
	R1 = 0x0000;
	[R_DebounceReg] = R1;
	[R_KeyBuf] = R1;
	[R_KeyStrobe] = R1;
	R1 = C_DebounceCnt;
	[R_DebounceCnt] = R1;
	retf;
	.endp
	
//****************************************************************
// Function    : F_System_Initial_User_IO
// Description : I/O initialization
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
F_System_Initial_User_IO: .proc
	R1 = [P_IOC_Dir]
	R1 |= 0x02FF
	[P_IOC_Dir] = R1
	R1 = [P_IOC_Attrib]
	R1 |= 0x02FF
	[P_IOC_Attrib] = R1	
	R1 = 0x0000;		
	[P_IOC_Buffer] = R1;
	retf;
	.endp	

//****************************************************************
// Function    : F_WatchdogClear
// Description : Clear watchdog register
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_WatchdogClear:	.proc
F_WatchdogClear:
	R1 = C_WDT_Clear;
	[P_WatchDog_Clear] = R1;
	retf;
	.endp

//****************************************************************
// Function    : F_System_ServiceLoop
// Description : Key scan and watchdog clear
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
 _System_ServiceLoop: .proc
F_System_ServiceLoop:
	call F_Key_DebounceCnt_Down;		// debounce counter countdown
	call F_Key_Scan_ServiceLoop;		// key scan
	call F_WatchdogClear;				// clear watchdog register
	retf;
	.endp;

//****************************************************************
// Function    : F_Key_DebounceCnt_Down
// Description : Debounce counter countdown
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
F_Key_DebounceCnt_Down:
	R1 = [R_DebounceCnt];
	jz	L_DebounceCntZero;
	R1 -= 0x0001;
	[R_DebounceCnt] = R1;
L_DebounceCntZero:
	retf;
	
//****************************************************************
// Function    : F_Key_Scan_ServiceLoop
// Description : Get Key code from key pad(8 x 1 key pad)
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : C_SCAN_8_Bits  = 1 , ScanKey 8bits
//               C_SCAN_8_Bits  = 0 , ScanKey 16bits  
//****************************************************************
F_Key_Scan_ServiceLoop:	.proc

	//R1 = [P_IOA_Data];				// get key data from IOA  
	R1 = [P_IOF_Data];					// get key data from IOB  

.IF C_SCAN_8_Bits					// Notice, User can't operate the IOA in interrupt, because it will change the IOA configration.
	R1 &= 0xFF00; 			          
.ELSE	
	R1 &= 0xFFFF; 
.ENDIF

	R2 = [R_DebounceReg];
	[R_DebounceReg] = R1;
	cmp R2, [R_DebounceReg];
	je ?L_KS_StableTwoSample;

	R1 = C_DebounceCnt;				   // debounce counter reset
	[R_DebounceCnt] = R1;
	retf;

?L_KS_StableTwoSample:
	R1 = [R_DebounceCnt];
	jz ?L_KS_StableOverDebounce;
	retf;

?L_KS_StableOverDebounce:
	[R_DebounceCnt] = R1;
	R2 = [R_DebounceReg];
	R1 = [R_KeyBuf];
	[R_KeyBuf] = R2;

.IF  C_SCAN_8_Bits	
	R1 ^= 0xFF00;    					
	R1 = R1 and [R_KeyBuf];
	R1 &= 0xFF00;
.ELSE
	R1 ^= 0xFFFF;    					
	R1 = R1 and [R_KeyBuf];
	R1 &= 0xFFFF;
.ENDIF							
	
	R1 |= [R_KeyStrobe];
	[R_KeyStrobe] = R1;
	retf;
	.endp


//****************************************************************
// Function    : F_SP_GetCh
// Description : Get Keycode
// Destory     : R1, R2
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
 _SP_GetCh:	.proc
F_SP_GetCh:
	R1 = [R_KeyStrobe];				// Get Key code
	R2 = 0x0000;					// Clear KeyStrobe for next key
	[R_KeyStrobe] = R2;
	retf;
	.endp
	
	
//=======================================================================================
// Function name: F_SYS_CounterDisable
// Purpose		: Disable 128Hz INT
// Parameter	: 
// Return		: None
// Destroy		: r1
//=======================================================================================
F_SYS_CounterDisable:
_F_SYS_CounterDisable:
	r1=(C_TimeBaseCFlag+C_TimeBaseCDis)
	[P_TimeBaseC_Ctrl]=r1
	retf
		
	
//=======================================================================================
// Function name: F_SYS_GetCount
// Purpose		: Get 128Hz Counter
// Parameter	: None
// Return		: R1 (Counter Value)
// Destroy		: R1
//=======================================================================================
F_SYS_GetCount:
_F_SYS_GetCount:
	R1=[R_128HzCounter]
	retf


//****************************************************************
// Function    : F_Delay1ms
// Description : Delay for Reset
// Destory     : R1
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
 _Delay1ms:
 	R1 = SP +3;
 	R1 = [R1++];
F_Delay1ms: .proc
    push R1,R2 to [SP];
	
?Loop_0:	
	R2 = 20000;
?Loop_1:
	R2 -= 0x0001;
	jnz ?Loop_1;
	R1 -= 0x0001;
	jnz ?Loop_0;
	
    pop R1,R2 from [SP];
	retf;
	.endp    


	