//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2009   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==========================================================================
// Program Name: SPIF.asm
// Applied body: GPL95100UA Series Body
// Programmer  : 
// Description : USB HID Mouse Service Code
// History version
// Rev #     Date       Who     Comments
// -----  -----------  ------   --------------------------------------------
// V1.0		2018/03/01  KC		Original

// -----  -----------  ------   --------------------------------------------

//**************************************************************************
// Header File Included Area
//**************************************************************************
#include "GPL951_Body.h"
#include ".\USB\USBD_Define.h"
#include ".\USB\USBD.h"


//**************************************************************************
// External Variable Declaration
//**************************************************************************
extern int HIDProtocol;
extern unsigned int R_Descriptor_Low;
extern unsigned int R_Descriptor_High;
extern unsigned char 	au8Buffer0[];
extern unsigned int au8buf_ptr;
extern unsigned int VendorCmdFlag;

void Read_EP0_FIFO(void);

//**************************************************************************
// Contant Defintion Area
//**************************************************************************

//**************************************************************************
// Variable Publication Area
//**************************************************************************

//**************************************************************************
// Function Call Publication Area
//**************************************************************************

//**************************************************************************
// RAM Definition Area
//**************************************************************************

//**************************************************************************
// CODE Definition Area
//**************************************************************************
void USB_ISR()
{ 
	int a;
	
	if (*P_USBD_INT){						//USB device interrupt event happened.
				
		if (*P_USBD_INT & BIT0){		// USB Device Receive Setup Packet
			*P_USBD_INT = BIT0; 			// Set EP0_State to EP0_Setup_Stage

			R_USB_State_Machine=EP0_SETUP_Stage;
		}
		
		else if	(*P_USBD_INT & BIT1){		// EP0 OUT Packet Clear OUT
			*P_USBD_INT = BIT1; 			// This bit is set when OUT PKT is load into fifo buffer
			*P_USBD_EPEvent = 0x0040;		// Then Enable EP0 Status Stage
		}
		
		else if(*P_USBD_INT & BIT2){
			USB_Status =  EP0_OUT_NACK;		// Then set USB_Status=EP0_OUT_NACK
			a=*P_USBD_INTEN;				// Disable EP0_IN_NACK INT
			*P_USBD_INTEN=a & 0xffef;
			a=*P_USBD_INTEN;
			*P_USBD_INTEN=a|0x0002;
			*P_USBD_INT = BIT2; 			// But Device reply NACK

			*P_USBD_EPEvent = 0x0002;		// Then Enable EP0 Status Stage
		}
				
		else if	(*P_USBD_INT & BIT3){		// EP0 IN Packet Clear INT
			*P_USBD_INT = BIT3; 			// This bit is set when IN PKT is read from the host
			*P_USBD_EPEvent = 0x0040;		// Then Enable EP0 Status Stage
		}
		
		else if	(*P_USBD_INT & BIT4){		// USB Host Send IN Token
			USB_Status =  EP0_IN_NACK;		// Then set USB_Status=EP0_IN_NACK
			a=*P_USBD_INTEN;				// Disable EP0_IN_NACK INT
			*P_USBD_INTEN=a & 0xffef;
			*P_USBD_INT = BIT4; 			// But Device reply NACK
		}
		
		else if	(*P_USBD_INT & BIT5){		// USB Host Start EP0 Status Stage, and
			*P_USBD_INT = BIT5; 			// Device has finished this stage.
				R_USB_State_Machine=0;
//			R_EP0DataCount = 0;				//huck: Modify to fix cable too long or connect with a long cable hub
		}
		else if (*P_USBD_INT & BIT12){
			R_USB_State_Machine_HID=INT_IN_Stage;
			USB_Status_HID=	INT_IN_NACK	;	
			a=*P_USBD_INTEN;				// Disable INT_IN_NACK INT
			*P_USBD_INTEN=a & 0xefff;
			*P_USBD_INT = BIT12;				// Clear Interrupt IN NACK Interrupt Flag
		}		
		else if	(*P_USBD_INT & BIT8){		// USB Host Send Bulk IN Token
								 			// But Device reply NACK
			USB_Status =  BULK_IN_NACK;			
			a=*P_USBD_INTEN;				// Disable Bulk_IN_NACK INT
			*P_USBD_INTEN=a & 0xfeff;
			*P_USBD_INT = BIT8;				// Clear Bulk IN NACK Interrupt Flag
		}		
		else if	(*P_USBD_INT & BIT10){		// USB Host Send Bulk OUT PKT
							 				// But Device reply NACK
			USB_Status =  BULK_OUT_NACK;	// Then set USB_Status=Bulk_OUT_NACK
			R_USB_State_Machine=SCSI_CBW_Stage;
			a=*P_USBD_INTEN;				// Disable Bulk_OUT_NACK INT
			*P_USBD_INTEN=a & 0xfbff;
			*P_USBD_INT = BIT10;			// Clear Bulk OUT Interrupt Flag
		}
		else if(*P_USBD_INT & BIT13	){		// USB Suspend
			a=*P_USBD_INTEN;
			a=a | 0x4000;					// Enable Resume Interrupt
			a=a & 0xdfff;					// Disable Suspend Interrupt		
			*P_USBD_INTEN=a;
			USBD_PlugIn = 0;
			*P_USBD_INT = BIT13;		
			USBDSuspend_CallBack();			//user could do something
		}		
		else if(*P_USBD_INT & BIT14){			// USB Resume
			*P_USBD_INT = BIT14;
			a=*P_USBD_INTEN;
			a=a | 0x2000;					// Enable Suspend Interrupt
			a=a & 0xbfff;					// Disable Resume Interrupt
			*P_USBD_INTEN=a;
			R_USB_Suspend=0;
		}		
		else if(*P_USBD_INT & BIT15){			// USB Reset
			*P_USBD_INT = BIT15; 	
			R_USB_Suspend=0;
			USBD_PlugIn = 1;
			USB_Reset();
		}
	
	}	
}

void USB_Reset(void)
{
	if (R_EP0DataCount)
		R_EP0DataCount = 0;

	R_USB_State_Machine=0;
	USB_Status=0;
	R_USB_Suspend=0;
	*P_USBD_EPBufClear = 0x000F;	//huck add
	*P_USBD_INTEN = 0x8429;
	data_send_flag = 0;
	gEP0InOutFlag = 0;
	
}

void USB_ServiceLoop(unsigned int unUseLoop)
{
	unsigned a;
	#ifdef print	
	init_uart();
	print1("\n\n\n\n\n/// SPL162001 USB DEVICE MSDC Program Start ///\n");
	#endif
	
	while(1)
	{	
		if(R_USB_State_Machine_HID)
		{
			if(USB_Status_HID==INT_IN_NACK)
				{										
					if(data_send_flag)
					{
						INT_IN_Data();		
						ClearMouseBuffer();							
						a=*P_USBD_INTEN;				// Enable INT_IN_NACK Interrupt
						a |= 0x1000;					// Used fo SCSI CBW Stage
						*P_USBD_INTEN=a;
						data_send_flag -= 1;
					}
					R_USB_State_Machine_HID=0;
				}
		}	
 		switch (R_USB_State_Machine)
		{
			
			case EP0_SETUP_Stage:
				R_USB_State_Machine=EP0_Data_Stage;
				Setup_Token();
			
				*P_USBD_EPEvent |= 0x4;
				*P_USBD_EPBufClear|=0x01;
				USB_Status=0x00;
				a=*P_USBD_INTEN;				// Enable EP0_IN_NACK INT
				if(gEP0InOutFlag==0)
				{
					a=*P_USBD_INTEN;				// Enable EP0_IN_NACK INT
					*P_USBD_INTEN=a|0x0010;
					*P_USBD_INT=0x0010;				// Clear EP0_IN_NACK	// added by Anson 2005/06/30 for verify test				
				}else
				{
					a=*P_USBD_INTEN;			// Enable EP0_OUT_NACK INT
					*P_USBD_INTEN=a|0x004;		//tony
					*P_USBD_INT=0x0004;			// Clear EP0_OUT_NACK	// added by Anson 2005/06/30 for verify test
					//gEP0InOutFlag=0;		
				}
				
				break;
			
			case EP0_Data_Stage:
				if(USB_Status==EP0_IN_NACK)		// When USB_Status = EP0 IN NACK
				{								// Start EP0 Data Stage
					Write_EP0_FIFO();					
					USB_Status=0x00;
				}
				else if(USB_Status==EP0_OUT_NACK)
				{	
					while((*P_USBD_EPEvent&0x04)!=0x04);			
					Read_EP0_FIFO();						
					USB_Status=0x00;
				}			
				break;
			
			default:
				*P_WatchDog_Clear = 0xA005;
				break;
		}
		
		if(R_USB_Suspend == 1)
		{
			break;
		}
// 		USB_ISR();		
		VendorCmdService();
		
		if (!unUseLoop)
			break;
	}
}		



void Setup_Token(void)
{
	unsigned int i;

	*P_USBD_EPEvent=0x0001;			// Clear Setup Packet Ready
	USB_Status=0x00;				// added by Anson 2005/06/28 for verify test
	if(*P_USBD_EP0RTR==0x80){
		switch((0xff00 & *P_USBD_EP0VR)){
			
			//////////////////	Get Device Descriptor	///////////////////
			case 0x0100:			// Device Descriptor
				i=Get_Descriptor_Length(Device_Descriptor_Table);
				if(*P_USBD_EP0LR > i)
					R_EP0DataCount=i;			// Assign Data Count
				else
					R_EP0DataCount=*P_USBD_EP0LR;
					if (!EP0_DataPointer_CallBack(Device_Descriptor_Table))
						DataPointer(Device_Descriptor_Table);				// Assign Data Pointer
					
					*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
//					EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
					#ifdef print
//					print1("Get Device Descriptor\n");
					#endif
				break;
				
			
			//////////////////	Get Config Descriptor	///////////////////
			case 0x0200:			// Configuration Descriptor
				
				i=*P_USBD_INTEN;				// Enable Suspend Interrupt
				i |= 0x2000;					//
				*P_USBD_INTEN=i;
		
				i=Get_Descriptor_Length(Config_Descriptor_Table);
				if(*P_USBD_EP0LR > i)
					R_EP0DataCount=i;
				else
					R_EP0DataCount=*P_USBD_EP0LR;
					
					if (!EP0_DataPointer_CallBack(Config_Descriptor_Table))
						DataPointer(Config_Descriptor_Table);				// Assign Data Pointer
					
					*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
					
//					EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
					#ifdef print	
//					print1("Get Config Descriptor\n");
					#endif
				break;
			
			
			//////////////////	Get String Descriptor	///////////////////
			case 0x0300:			// String Descriptor
				switch(*P_USBD_EP0VR & 0x00ff){
					case 0x0000:
						i=Get_Descriptor_Length(String0_Descriptor_Table);
						if(*P_USBD_EP0LR > i)
							R_EP0DataCount=i;
						else
							R_EP0DataCount=*P_USBD_EP0LR;
	
						if (!EP0_DataPointer_CallBack(String0_Descriptor_Table))
							DataPointer(String0_Descriptor_Table);				// Assign Data Pointer
						
						*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
						
//						EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
						#ifdef print	
//						print1("Get String 0 Descriptor\n");
						#endif
						break;
						
					case 0x0001:
						i=Get_Descriptor_Length(String1_Descriptor_Table);
						if(*P_USBD_EP0LR > i)
							R_EP0DataCount=i;
						else
							R_EP0DataCount=*P_USBD_EP0LR;
							
						if (!EP0_DataPointer_CallBack(String1_Descriptor_Table))					
							DataPointer(String1_Descriptor_Table);				// Assign Data Pointer
						
						*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
						
//						EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
						#ifdef print	
//						print1("Get String 1 Descriptor\n");
						#endif
						break;
						
					case 0x0002:
						i=Get_Descriptor_Length(String2_Descriptor_Table);
						if(*P_USBD_EP0LR > i)
							R_EP0DataCount=i;
						else
							R_EP0DataCount=*P_USBD_EP0LR;
					
						if (!EP0_DataPointer_CallBack(String2_Descriptor_Table))						
							DataPointer(String2_Descriptor_Table);				// Assign Data Pointer
						
						*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
						
//						EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
						#ifdef print	
//						print1("Get String 2 Descriptor\n");
						#endif
						break;
						
					case 0x0003:
						i=Get_Descriptor_Length(String3_Descriptor_Table);
						if(*P_USBD_EP0LR > i)
							R_EP0DataCount=i;
						else
							R_EP0DataCount=*P_USBD_EP0LR;
					
						if (!EP0_DataPointer_CallBack(String3_Descriptor_Table))
							DataPointer(String3_Descriptor_Table);				// Assign Data Pointer
						
						*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
						
//						EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
						#ifdef print	
//						print1("Get String 3 Descriptor\n");
						#endif
						break;
						
					case 0x0004:
						i=Get_Descriptor_Length(String4_Descriptor_Table);
						if(*P_USBD_EP0LR > i)
							R_EP0DataCount=i;
						else
							R_EP0DataCount=*P_USBD_EP0LR;

						if (!EP0_DataPointer_CallBack(String4_Descriptor_Table))					
							DataPointer(String4_Descriptor_Table);				// Assign Data Pointer
						
						*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	// added by Anson 2005/06/28 for verify test
						
//						EP0_State=EP0_Data_Stage;					// Change USB State to EP0 Data Stage
						#ifdef print	
//						print1("Get String 4 Descriptor\n");
						#endif
						break;
					default:
						#ifdef print	
						print1("Not Support    Check (String)\n");
						#endif
						NotSupport();
						break;	
				}
				
				break;	
			default:
				NotSupport();
				#ifdef print	
				print1("Not Support    Check\n");
				#endif
				break;
		
		
		
		}
	}
	else if(*P_USBD_EP0RTR==0xA1 && *P_USBD_EP0RR==0xFE && *P_USBD_EP0VR==0x00){
		R_EP0DataCount=*P_USBD_EP0LR;
		DataPointer(MaxLUN_Table);
		#ifdef print	
		print1("Get Max LUN\n");
		#endif
		*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK
		while((*P_USBD_EPEvent & 0x0020) == 0);
		Write_EP0_FIFO();
		*P_USBD_EPEvent = 0x0040;				// Then Enable EP0 Status Stage
		
		//////////////////////////////////////////////////////////////
		// Change to USB MSDC SCSI Stage
//		SCSI_State=SCSI_CBW_Stage;
		R_USB_State_Machine=SCSI_CBW_Stage;
		
		*P_USBD_EPEvent=0x2000;			// Clear Bulk OUT NACK
		
		i=*P_USBD_INTEN;				// Enable Bulk OUT NACK Interrupt
		i |= 0x0400;					// Used fo SCSI CBW Stage
		*P_USBD_INTEN=i;
	}
	else if(*P_USBD_EP0RTR==0x21 && *P_USBD_EP0RR==0xFF && *P_USBD_EP0VR==0x00){ //Reset Recovery
		for(i = 0;i < 64;i++)
		{
			*P_USBD_BOData = 0;
			*P_USBD_BIData = 0; 
		} 
		*P_USBD_EPEvent=0x0008;						// Clear EP0 IN NACK
		R_EP0DataCount = 0;
//		Write_EP0_FIFO();
//		*P_USBD_EPEvent = 0x0040;				// Then Enable EP0 Status Stage
	}
	//set config
	else if(*P_USBD_EP0RTR == 0 && *P_USBD_EP0RR == 9 && *P_USBD_EP0VR == 1)
	{
		//NotSupport();
		*P_USBD_EPEvent=0x0008;						// Clear EP0 IN NACK
		R_EP0DataCount = 0;
	}
	//set idle
	else if(*P_USBD_EP0RTR == 0x21 && *P_USBD_EP0RR == 0x0a && *P_USBD_EP0VR == 0)
	{
		*P_USBD_EPEvent=0x0008;						// Clear EP0 IN NACK
		R_EP0DataCount = 0;
	}
	else if((*P_USBD_EP0RTR == 0xa1)&&(*P_USBD_EP0RR == 0x03)) //HID GET_Protocol
	{
		R_EP0DataCount=*P_USBD_EP0LR;
		DataPointer(HIDProtocol_Table);	
		*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK
		while((*P_USBD_EPEvent & 0x0020) == 0);
			Write_EP0_FIFO();
		*P_USBD_EPEvent = 0x0040;				// Then Enable EP0 Status Stage
		R_USB_State_Machine=SCSI_CBW_Stage;		
		*P_USBD_EPEvent=0x2000;			// Clear Bulk OUT NACK		
		i=*P_USBD_INTEN;				// Enable Bulk OUT NACK Interrupt
		i |= 0x0400;					// Used fo SCSI CBW Stage
		*P_USBD_INTEN=i;		
	}
	else if((*P_USBD_EP0RTR == 0x21)&&(*P_USBD_EP0RR == 0x0B)) //HID SET_Protocol
	{
		HIDProtocol = *P_USBD_EP0VR;
	}
	else if(*P_USBD_EP0RTR == 0x81 && *P_USBD_EP0RR == 6 && *P_USBD_EP0VR == 0x2100)
	{
		i=Get_Descriptor_Length(HIDDescriptor_Table);
		if(*P_USBD_EP0LR > i)
			R_EP0DataCount=i;			// Assign Data Count
		else
			R_EP0DataCount=*P_USBD_EP0LR;
		if (!EP0_DataPointer_CallBack(HIDDescriptor_Table))
			DataPointer(HIDDescriptor_Table);				// Assign Data Pointer		
		*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK	
		*P_USBD_EPEvent=0x8000;	
	}
	//get interface descriptor	
	else if(*P_USBD_EP0RTR == 0x81 && *P_USBD_EP0RR == 6 && *P_USBD_EP0VR == 0x2200){
		i=Get_Descriptor_Length(KeyBoardReportDescriptor_Table);
		if(*P_USBD_EP0LR > i)
			R_EP0DataCount=i;			// Assign Data Count
		else
			R_EP0DataCount=*P_USBD_EP0LR;
		if (!EP0_DataPointer_CallBack(KeyBoardReportDescriptor_Table))
			DataPointer(KeyBoardReportDescriptor_Table);				// Assign Data Pointer
			
		*P_USBD_EPEvent=0x0020;						// Clear EP0 IN NACK		
		*P_USBD_EPEvent=0x8000;			// Clear int in NACK	
	}
	
	//set report
	else if(*P_USBD_EP0RTR == 0x21 && *P_USBD_EP0RR == 0x09 && (*P_USBD_EP0VR&0xFF00) == 0x0200)
	{
		switch(*P_USBD_EP0VR&0xFF)
		{
			case 0xF0:		//PC send 4 byte random key					
					VendorCmdFlag = 0x02F0;
					au8buf_ptr = 0;
					R_EP0DataCount = *P_USBD_EP0LR;
				break;
			case 0xF1:		//Unlock device	
					VendorCmdFlag = 0x02F1;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;
				break;
			case 0x01:		//reserve write
					VendorCmdFlag = 0x0201;
					au8buf_ptr = 0;
					R_EP0DataCount = *P_USBD_EP0LR;
				break;
			case 0x11:		//Vendor command write flash parameters
					VendorCmdFlag = 0x0211;
					au8buf_ptr = 0;
					R_EP0DataCount = *P_USBD_EP0LR;
				break;
			case 0x12:		//Vendor command write flash data
					VendorCmdFlag = 0x0212;
					au8buf_ptr = 0;
					R_EP0DataCount = *P_USBD_EP0LR;
				break;
			default:
				break;																				
		}
		*P_USBD_EPEvent=0x0008;						// Clear EP0 OUT NACKS
		gEP0InOutFlag = 1;
	}

	//get report
	else if(*P_USBD_EP0RTR == 0xA1 && *P_USBD_EP0RR == 0x01 && (*P_USBD_EP0VR&0xFF00) == 0x0100)
	{
		R_Descriptor_Low = (unsigned int)&au8Buffer0;
		R_Descriptor_High = 0;		
		switch(*P_USBD_EP0VR&0xFF)
		{
			case 0xF0:		//PC read 2 byte key number
					VendorCmdFlag = 0x81F0;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;

				break;
			case 0xF1:		//Lock device	
					VendorCmdFlag = 0x81F1;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;
				break;
			case 0x01:		//reserve read
					VendorCmdFlag = 0x8101;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;
				break;
			case 0xFA:		//Get IC version
					VendorCmdFlag = 0x81FA;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;
				break;				
			case 0x12:		//Vendor command read flash data
					VendorCmdFlag = 0x8112;
					R_EP0DataCount = *P_USBD_EP0LR;
					au8buf_ptr = 0;
				break;
			default:
				break;																		
		}
		*P_USBD_EPEvent=0x0008;						// Clear EP0 OUT NACKS
		gEP0InOutFlag = 0;
	}	
	
	else{
		NotSupport();
		#ifdef print	
		print1("Not Support    Other Setup Command\n");
		#endif
		
	}	
	
	
}

void Write_EP0_FIFO()
{
	int a;
	
	if(R_EP0DataCount==0){
		// reply Null Packet
		Send_Null();
		#ifdef print	
		print1("Send Null Packet\n");
		#endif
		return;
	}
	
	while (R_EP0DataCount)		//huck: Modify to fix cable too long issue or connect with a long cable hub
	{
		if(R_EP0DataCount >= 8)
		{
			Send_8_Bytes();
	
			a=*P_USBD_INTEN;			// Enable EP0_IN_NACK INT
			*P_USBD_INTEN=a|0x0010;
			*P_USBD_INT=0x0010;			// Clear EP0_IN_NACK	// added by Anson 2005/06/30 for verify test

		}
		else if(R_EP0DataCount <8)
		{
			Send_Remain_Bytes();
		}
	}	
	return;
}

//tony
void Read_EP0_FIFO()
{
	int a,i;
	
	while (R_EP0DataCount)		//huck: Modify to fix cable too long issue or connect with a long cable hub
	{
		if(R_EP0DataCount >= 8)
		{
			while((*P_USBD_EPEvent&0x04)!=0x04);
			for(i=0;i<8;i++)
			{
				au8Buffer0[au8buf_ptr] = *P_USBD_EP0Data;
				au8buf_ptr++;
				R_EP0DataCount--;
			}
			
			*P_USBD_EPEvent |= 0x4;
			*P_USBD_EPBufClear|=0x01;
			a=*P_USBD_INTEN;			// Enable EP0_OUT_NACK INT
			*P_USBD_INTEN=a|0x004;		//tony
			*P_USBD_INT=0x0004;			// Clear EP0_OUT_NACK	// added by Anson 2005/06/30 for verify test				
		}
		else if(R_EP0DataCount <8)
		{
			for(i=0;i<R_EP0DataCount;i++)
			{
				au8Buffer0[au8buf_ptr] = *P_USBD_EP0Data;
				au8buf_ptr++;
			}
			R_EP0DataCount = 0;
			gEP0InOutFlag=0;	
			a=*P_USBD_INTEN;			// Enable EP0_IN_NACK INT
			*P_USBD_INTEN=a|0x0010;
			*P_USBD_INT=0x0010;			// Clear EP0_IN_NACK	// added by Anson 2005/06/30 for verify test
			
			a=*P_USBD_INTEN;			// Enable EP0_IN_NACK INT	
		}
		*P_USBD_EPBufClear|=0x01;		//clear EP0 data buf
		if(R_EP0DataCount==0)
		{
			gEP0InOutFlag = 0;
			VendorCmdFlag |= 0x8000;	//data end
		}		
	}	
	return;
}

void NotSupport()
{
	int a;
	
	// reply stall	
	*P_USBD_EPSetStall=0x0001;	// Enable EP0 Stall
				
	a=*P_USBD_EPStall & 0x0100;		// wait until device reply stall 

	ulTimeout = 0x2FFFF;
	while(a==0)
	{
		if (!(ulTimeout--))
			break;																
		a=*P_USBD_EPStall & 0x0100;
	}
	*P_USBD_EPStall=0x0100;			// clear EP0 Stall Flag
					
	*P_USBD_EPSetStall=0x0000;	// Disable EP0 Stall
	R_USB_State_Machine=0;
}




